"""
Configuration management for DevEnv Manager
"""

from pathlib import Path
from typing import Any, Dict

import yaml


class Config:
    def __init__(self):
        self.home_dir = Path.home()
        self.config_dir = self.home_dir / ".devenv"
        self.snapshots_dir = self.config_dir / "snapshots"
        self.config_file = self.config_dir / "config.yaml"

        # Create directories if they don't exist
        self.config_dir.mkdir(exist_ok=True)
        self.snapshots_dir.mkdir(exist_ok=True)

        # Load or create default config
        self.settings = self._load_config()

    def _load_config(self) -> Dict[str, Any]:
        """Load configuration from file or create default"""
        if self.config_file.exists():
            with open(self.config_file, "r") as f:
                return yaml.safe_load(f) or {}
        else:
            # Create default config
            default_config = {
                "version": "0.1.0",
                "capture": {
                    "include_dotfiles": True,
                    "include_packages": True,
                    "include_vscode_extensions": True,
                    "backup_ssh_keys": False,  # Security: disabled by default
                },
                "storage": {
                    "local_only": True,
                    "git_sync": False,
                    "cloud_providers": [],
                },
            }
            self.save_config(default_config)
            return default_config

    def save_config(self, config: Dict[str, Any] = None):
        """Save configuration to file"""
        config_to_save = config or self.settings
        with open(self.config_file, "w") as f:
            yaml.dump(config_to_save, f, default_flow_style=False)

    def get(self, key: str, default=None):
        """Get configuration value"""
        keys = key.split(".")
        value = self.settings
        for k in keys:
            value = value.get(k, {}) if isinstance(value, dict) else default
        return value if value != {} else default


# Global config instance
config = Config()



"""
System detection utilities for DevEnv Manager
"""

import os
import subprocess
from pathlib import Path
from typing import Any, Dict, List


class SystemDetector:
    def __init__(self):
        self.home_dir = Path.home()

    def detect_packages(self) -> Dict[str, List[str]]:
        """Detect installed packages"""
        packages = {"apt": [], "snap": [], "flatpak": [], "pip": []}

        # APT packages (manually installed only)
        try:
            result = subprocess.run(
                ["apt-mark", "showmanual"],
                capture_output=True,
                text=True,
                check=True,
            )
            packages["apt"] = (
                result.stdout.strip().split("\n")
                if result.stdout.strip()
                else []
            )
        except (subprocess.CalledProcessError, FileNotFoundError):
            pass

        # Snap packages
        try:
            result = subprocess.run(
                ["snap", "list"], capture_output=True, text=True, check=True
            )
            lines = result.stdout.strip().split("\n")[1:]  # Skip header
            packages["snap"] = [
                line.split()[0] for line in lines if line.strip()
            ]
        except (subprocess.CalledProcessError, FileNotFoundError):
            pass

        # Flatpak packages
        try:
            result = subprocess.run(
                ["flatpak", "list", "--app", "--columns=application"],
                capture_output=True,
                text=True,
                check=True,
            )
            packages["flatpak"] = (
                result.stdout.strip().split("\n")
                if result.stdout.strip()
                else []
            )
        except (subprocess.CalledProcessError, FileNotFoundError):
            pass

        # Global pip packages
        try:
            result = subprocess.run(
                ["pip", "list", "--format=freeze"],
                capture_output=True,
                text=True,
                check=True,
            )
            packages["pip"] = (
                result.stdout.strip().split("\n")
                if result.stdout.strip()
                else []
            )
        except (subprocess.CalledProcessError, FileNotFoundError):
            pass

        return packages

    def detect_dotfiles(self) -> Dict[str, str]:
        """Detect important dotfiles"""
        dotfiles = {}
        important_dotfiles = [
            ".bashrc",
            ".bash_profile",
            ".zshrc",
            ".profile",
            ".vimrc",
            ".gitconfig",
            ".ssh/config",
        ]

        for dotfile in important_dotfiles:
            file_path = self.home_dir / dotfile
            if file_path.exists() and file_path.is_file():
                try:
                    with open(file_path, "r") as f:
                        dotfiles[dotfile] = f.read()
                except Exception as e:
                    print(f"Warning: Could not read {dotfile}: {e}")

        return dotfiles

    def detect_vscode_extensions(self) -> List[str]:
        """Detect VS Code extensions"""
        extensions = []
        try:
            result = subprocess.run(
                ["code", "--list-extensions"],
                capture_output=True,
                text=True,
                check=True,
            )
            extensions = (
                result.stdout.strip().split("\n")
                if result.stdout.strip()
                else []
            )
        except (subprocess.CalledProcessError, FileNotFoundError):
            pass

        return extensions

    def get_system_info(self) -> Dict[str, Any]:
        """Get basic system information"""
        return {
            "os": os.uname().sysname,
            "kernel": os.uname().release,
            "architecture": os.uname().machine,
            "hostname": os.uname().nodename,
            "python_version": os.sys.version.split()[0],
            "shell": os.environ.get("SHELL", "unknown"),
            "user": os.environ.get("USER", "unknown"),
        }


# Global detector instance
detector = SystemDetector()



#!/usr/bin/env python3
"""
DevEnv Manager - Main CLI Interface
"""
from datetime import datetime

import click
from rich.console import Console
from rich.panel import Panel
from rich.table import Table

from ..core.detector import detector
from ..core.restorer import restorer
from ..core.sync import git_sync
from ..storage.local import storage

console = Console()


@click.group()
@click.version_option(version="0.1.0")
def cli():
    """DevEnv Manager - Manage your development environments with ease!"""


@cli.command()
def init():
    """Initialize DevEnv Manager in current directory"""
    console.print(
        Panel.fit(
            "[bold green]🚀 DevEnv Manager initialized "
            "successfully![/bold green]\n"
            f"Config stored in: {storage.snapshots_dir.parent}",
            title="Init Complete",
        )
    )


@cli.command()
@click.argument("name")
def capture(name):
    """Capture current development environment"""
    console.print(
        f"[bold blue]📸 Capturing environment: {name}[/bold blue]"
    )

    with console.status("[bold green]Detecting system configuration..."):
        # Collect system data
        data = {
            "system_info": detector.get_system_info(),
            "packages": detector.detect_packages(),
            "dotfiles": detector.detect_dotfiles(),
            "vscode_extensions": detector.detect_vscode_extensions(),
        }

    # Save snapshot
    if storage.save_snapshot(name, data):
        # Show summary
        table = Table(title="Capture Summary")
        table.add_column("Component", style="cyan")
        table.add_column("Count", style="green")

        table.add_row("APT Packages", str(len(data["packages"]["apt"])))
        table.add_row("Snap Packages", str(len(data["packages"]["snap"])))
        table.add_row(
            "Flatpak Packages", str(len(data["packages"]["flatpak"]))
        )
        table.add_row("PIP Packages", str(len(data["packages"]["pip"])))
        table.add_row("Dotfiles", str(len(data["dotfiles"])))
        table.add_row(
            "VS Code Extensions", str(len(data["vscode_extensions"]))
        )

        console.print(table)
        console.print(
            f"[green]✓ Environment '{name}' captured "
            "successfully![/green]"
        )
    else:
        console.print("[red]✗ Failed to capture environment![/red]")


@cli.command()
def list():
    """List all captured environments"""
    snapshots = storage.list_snapshots()

    if not snapshots:
        console.print(
            "[yellow]📋 No environments found. Use 'devenv capture "
            "<name>' to create one.[/yellow]"
        )
        return

    table = Table(title="Available Environments")
    table.add_column("Name", style="cyan")
    table.add_column("Created", style="green")
    table.add_column("File", style="dim")

    for snapshot in snapshots:
        created_date = datetime.fromisoformat(
            snapshot["created_at"]
        ).strftime("%Y-%m-%d %H:%M")
        table.add_row(snapshot["name"], created_date, snapshot["file"])

    console.print(table)


@cli.command()
@click.argument("name")
def show(name):
    """Show details of a captured environment"""
    data = storage.load_snapshot(name)

    if not data:
        console.print(f"[red]❌ Environment '{name}' not found![/red]")
        return

    console.print(f"[bold cyan]📋 Environment Details: {name}[/bold cyan]")

    # System info
    if "system_info" in data:
        info_table = Table(title="System Information")
        info_table.add_column("Property", style="cyan")
        info_table.add_column("Value", style="green")

        for key, value in data["system_info"].items():
            info_table.add_row(key.replace("_", " ").title(), str(value))

        console.print(info_table)

    # Packages summary
    if "packages" in data:
        pkg_table = Table(title="Packages Summary")
        pkg_table.add_column("Type", style="cyan")
        pkg_table.add_column("Count", style="green")

        for pkg_type, packages in data["packages"].items():
            if packages:
                pkg_table.add_row(pkg_type.upper(), str(len(packages)))

        console.print(pkg_table)


@cli.command()
@click.argument("name")
@click.option(
    "--dry-run",
    is_flag=True,
    help="Show what would be done without making changes",
)
@click.option("--force", is_flag=True, help="Skip confirmation prompts")
def restore(name, dry_run, force):
    """Restore a captured environment"""
    # Load snapshot data
    data = storage.load_snapshot(name)

    if not data:
        console.print(f"[red]❌ Environment '{name}' not found![/red]")
        return

    console.print(
        f"[bold magenta]🔄 Restoring environment: "
        f"{name}[/bold magenta]"
    )

    # Show what will be restored
    if "packages" in data:
        pkg_table = Table(title="Packages to Restore")
        pkg_table.add_column("Type", style="cyan")
        pkg_table.add_column("Count", style="green")

        total_packages = 0
        for pkg_type, packages in data["packages"].items():
            if packages:
                pkg_table.add_row(pkg_type.upper(), str(len(packages)))
                total_packages += len(packages)

        console.print(pkg_table)

        if not dry_run and not force:
            console.print(
                f"\n[yellow]⚠️  This will install "
                f"{total_packages} packages and may modify your "
                "system.[/yellow]"
            )
            if not click.confirm("Do you want to continue?"):
                console.print("[yellow]Restore cancelled.[/yellow]")
                return

    # Perform restoration
    success = restorer.restore_environment(data, dry_run=dry_run)

    if success:
        if dry_run:
            console.print(
                "[green]✓ Dry run completed successfully![/green]"
            )
        else:
            console.print(
                "[green]✓ Environment restored " "successfully![/green]"
            )
    else:
        console.print(
            "[red]✗ Some errors occurred during " "restoration![/red]"
        )


@cli.command()
def status():
    """Show current system status and available packages"""
    console.print("[bold cyan]📊 System Status[/bold cyan]")

    with console.status("[bold green]Analyzing system..."):
        system_info = detector.get_system_info()
        packages = detector.detect_packages()

    # System info table
    info_table = Table(title="System Information")
    info_table.add_column("Property", style="cyan")
    info_table.add_column("Value", style="green")

    for key, value in system_info.items():
        info_table.add_row(key.replace("_", " ").title(), str(value))

    console.print(info_table)

    # Packages summary
    pkg_table = Table(title="Installed Packages")
    pkg_table.add_column("Type", style="cyan")
    pkg_table.add_column("Count", style="green")

    for pkg_type, pkg_list in packages.items():
        if pkg_list:
            pkg_table.add_row(pkg_type.upper(), str(len(pkg_list)))

    console.print(pkg_table)


@cli.command()
@click.argument("name")
@click.option("--force", is_flag=True, help="Skip confirmation prompt")
def delete(name, force):
    """Delete a captured environment"""
    # Check if environment exists
    data = storage.load_snapshot(name)
    if not data:
        console.print(f"[red]❌ Environment '{name}' not found![/red]")
        return

    # Confirmation
    if not force:
        console.print(
            f"[yellow]⚠️  This will permanently delete "
            f"environment '{name}'[/yellow]"
        )
        if not click.confirm("Are you sure?"):
            console.print("[yellow]Delete cancelled.[/yellow]")
            return

    # Delete the environment
    if storage.delete_snapshot(name):
        console.print(
            f"[green]✓ Environment '{name}' deleted "
            "successfully![/green]"
        )
    else:
        console.print(
            f"[red]✗ Failed to delete environment " f"'{name}'![/red]"
        )


@cli.command()
@click.argument("name")
@click.argument("output_file", type=click.Path())
def export(name, output_file):
    """Export an environment to a file"""
    data = storage.load_snapshot(name)

    if not data:
        console.print(f"[red]❌ Environment '{name}' not found![/red]")
        return

    try:
        import json

        with open(output_file, "w") as f:
            json.dump(data, f, indent=2)

        console.print(
            f"[green]✓ Environment '{name}' exported to "
            f"{output_file}![/green]"
        )
    except Exception as e:
        console.print(f"[red]✗ Failed to export environment: {e}[/red]")


@cli.command()
@click.argument("input_file", type=click.Path(exists=True))
@click.option(
    "--name",
    help="Name for the imported environment "
    "(defaults to original name)",
)
def import_env(input_file, name):
    """Import an environment from a file"""
    try:
        import json

        with open(input_file, "r") as f:
            data = json.load(f)

        # Use provided name or original name
        env_name = name or data.get("metadata", {}).get(
            "name", "imported-env"
        )

        # Check if already exists
        if storage.load_snapshot(env_name):
            console.print(
                f"[yellow]⚠️  Environment '{env_name}' already "
                "exists![/yellow]"
            )
            if not click.confirm("Overwrite existing environment?"):
                console.print("[yellow]Import cancelled.[/yellow]")
                return

        # Save imported environment
        if storage.save_snapshot(env_name, data):
            console.print(
                f"[green]✓ Environment imported as "
                f"'{env_name}'![/green]"
            )
        else:
            console.print("[red]✗ Failed to import environment![/red]")

    except Exception as e:
        console.print(f"[red]✗ Failed to import environment: {e}[/red]")


@cli.command()
@click.argument("env1")
@click.argument("env2")
def diff(env1, env2):
    """Compare two environments and show differences"""
    # Load both environments
    data1 = storage.load_snapshot(env1)
    data2 = storage.load_snapshot(env2)

    if not data1:
        console.print(f"[red]❌ Environment '{env1}' not found![/red]")
        return

    if not data2:
        console.print(f"[red]❌ Environment '{env2}' not found![/red]")
        return

    console.print(f"[bold cyan]🔍 Comparing {env1} vs {env2}[/bold cyan]")

    # Compare packages
    if "packages" in data1 and "packages" in data2:
        _compare_packages(data1["packages"], data2["packages"], env1, env2)

    # Compare dotfiles
    if "dotfiles" in data1 and "dotfiles" in data2:
        _compare_dotfiles(data1["dotfiles"], data2["dotfiles"], env1, env2)

    # Compare VS Code extensions
    if "vscode_extensions" in data1 and "vscode_extensions" in data2:
        _compare_extensions(
            data1["vscode_extensions"],
            data2["vscode_extensions"],
            env1,
            env2,
        )


def _compare_packages(pkg1, pkg2, name1, name2):
    """Compare package lists between two environments"""
    console.print("\n[bold]📦 Package Differences:[/bold]")

    for pkg_type in ["apt", "snap", "flatpak", "pip"]:
        set1 = set(pkg1.get(pkg_type, []))
        set2 = set(pkg2.get(pkg_type, []))

        only_in_1 = set1 - set2
        only_in_2 = set2 - set1

        if only_in_1 or only_in_2:
            console.print(f"\n[cyan]{pkg_type.upper()} packages:[/cyan]")

            if only_in_1:
                console.print(
                    f"  Only in {name1}: {len(only_in_1)} " "packages"
                )
                for pkg in sorted(list(only_in_1)[:5]):  # Show first 5
                    console.print(f"    - {pkg}")
                if len(only_in_1) > 5:
                    console.print(f"    ... and {len(only_in_1) - 5} more")

            if only_in_2:
                console.print(
                    f"  Only in {name2}: {len(only_in_2)} " "packages"
                )
                for pkg in sorted(list(only_in_2)[:5]):  # Show first 5
                    console.print(f"    + {pkg}")
                if len(only_in_2) > 5:
                    console.print(f"    ... and {len(only_in_2) - 5} more")


def _compare_dotfiles(dotfiles1, dotfiles2, name1, name2):
    """Compare dotfiles between two environments"""
    console.print("\n[bold]📝 Dotfile Differences:[/bold]")

    set1 = set(dotfiles1.keys())
    set2 = set(dotfiles2.keys())

    only_in_1 = set1 - set2
    only_in_2 = set2 - set1
    common = set1 & set2

    if only_in_1:
        console.print(f"  Only in {name1}: {', '.join(sorted(only_in_1))}")

    if only_in_2:
        console.print(f"  Only in {name2}: {', '.join(sorted(only_in_2))}")

    # Check for content differences in common files
    different_content = []
    for filename in common:
        if dotfiles1[filename] != dotfiles2[filename]:
            different_content.append(filename)

    if different_content:
        console.print(
            f"  Different content: "
            f"{', '.join(sorted(different_content))}"
        )


def _compare_extensions(ext1, ext2, name1, name2):
    """Compare VS Code extensions between two environments"""
    console.print("\n[bold]🔌 VS Code Extension Differences:[/bold]")

    set1 = set(ext1)
    set2 = set(ext2)

    only_in_1 = set1 - set2
    only_in_2 = set2 - set1

    if only_in_1:
        console.print(f"  Only in {name1}: {len(only_in_1)} extensions")
        for ext in sorted(list(only_in_1)[:3]):
            console.print(f"    - {ext}")
        if len(only_in_1) > 3:
            console.print(f"    ... and {len(only_in_1) - 3} more")

    if only_in_2:
        console.print(f"  Only in {name2}: {len(only_in_2)} extensions")
        for ext in sorted(list(only_in_2)[:3]):
            console.print(f"    + {ext}")
        if len(only_in_2) > 3:
            console.print(f"    ... and {len(only_in_2) - 3} more")


@cli.command()
def clean():
    """Clean up old backup files and temporary data"""
    cleaned_files = 0

    # Clean up dotfile backups older than 30 days
    import time
    from pathlib import Path

    home_dir = Path.home()
    current_time = time.time()

    for backup_file in home_dir.glob("*/.devenv-backup"):
        try:
            file_age = current_time - backup_file.stat().st_mtime
            if file_age > (30 * 24 * 3600):  # 30 days
                backup_file.unlink()
                cleaned_files += 1
        except Exception:
            pass

    console.print(
        f"[green]✓ Cleaned up {cleaned_files} old backup " "files[/green]"
    )


# =====================
# SYNC COMMANDS
# =====================


@cli.group()
def sync():
    """Git-based synchronization commands"""


@sync.command()
@click.argument("repo_url")
@click.option("--branch", default="main", help="Git branch to use")
def setup(repo_url, branch):
    """Setup git sync with a remote repository"""
    console.print(f"[blue]🔧 Setting up git sync with {repo_url}[/blue]")

    if git_sync.setup_sync(repo_url, branch):
        console.print(
            Panel.fit(
                f"[bold green]Git sync setup complete![/bold green]\n\n"
                f"Repository: {repo_url}\n"
                f"Branch: {branch}\n\n"
                f"Use 'devenv sync push' to upload environments\n"
                f"Use 'devenv sync pull' to download environments",
                title="Sync Ready",
            )
        )
    else:
        console.print("[red]✗ Failed to setup git sync[/red]")


@sync.command()
@click.option(
    "--env", "-e", multiple=True, help="Specific environments to push"
)
def push(env):
    """Push environments to remote repository

    Examples:
      devenv sync push                    # Push all environments
      devenv sync push -e myenv          # Push specific environment
      devenv sync push -e env1 -e env2   # Push multiple environments
    """
    # Construir lista de ambientes (evitar list() direto por bug do Click)
    if env and len(env) > 0:
        env_list = []
        for item in env:
            env_list.append(item)
        console.print(
            f"[blue]📤 Pushing specific environments: "
            f"{', '.join(env_list)}[/blue]"
        )
    else:
        env_list = None
        console.print("[blue]📤 Pushing all environments...[/blue]")

    # Executar push
    result = git_sync.push_environments(env_list)

    # Mostrar resultado
    if result:
        if env_list:
            console.print(
                f"[green]✓ Successfully pushed "
                f"{len(env_list)} specific "
                "environments[/green]"
            )
        else:
            console.print(
                "[green]✓ Successfully pushed all " "environments[/green]"
            )
    else:
        console.print("[red]✗ Push operation failed[/red]")


@sync.command()
def pull():
    """Pull environments from remote repository"""
    git_sync.pull_environments()


@sync.command()
def sync_status():
    """Show git sync status"""
    status_info = git_sync.sync_status()

    if not status_info.get("enabled"):
        console.print("[yellow]📡 Git sync not configured[/yellow]")
        console.print("Run 'devenv sync setup <repo-url>' to get started")
        return

    # Create status table
    table = Table(title="Git Sync Status")
    table.add_column("Property", style="cyan")
    table.add_column("Value", style="green")

    table.add_row(
        "Status", "✓ Enabled" if status_info["enabled"] else "✗ Disabled"
    )

    if "remote_url" in status_info:
        table.add_row("Remote URL", status_info["remote_url"])

    if "branch" in status_info:
        table.add_row("Branch", status_info["branch"])

    if "uncommitted_changes" in status_info:
        changes_status = (
            "Yes" if status_info["uncommitted_changes"] else "No"
        )
        table.add_row("Uncommitted Changes", changes_status)

    if "last_commit" in status_info and status_info["last_commit"]:
        commit = status_info["last_commit"]
        table.add_row(
            "Last Commit", f"{commit['hash']} - {commit['message']}"
        )
        table.add_row("Commit Date", commit["date"])

    if "sync_dir" in status_info:
        table.add_row("Sync Directory", status_info["sync_dir"])

    console.print(table)

    if status_info.get("error"):
        console.print(f"[red]Error: {status_info['error']}[/red]")


if __name__ == "__main__":
    cli()



"""Test configuration module"""

import shutil
import tempfile
from pathlib import Path

from devenv.core.config import Config


def test_config_creation():
    """Test config initialization"""
    with tempfile.TemporaryDirectory() as temp_dir:
        # Mock home directory
        config = Config()
        assert config.settings is not None


def test_config_default_values():
    """Test default configuration values"""
    config = Config()
    assert config.get("version") == "0.1.0"
    assert config.get("capture.include_dotfiles") is True



"""
Local storage management for DevEnv Manager
"""

import json
from datetime import datetime
from typing import Any, Dict, List

from ..core.config import config


class LocalStorage:
    def __init__(self):
        self.snapshots_dir = config.snapshots_dir

    def save_snapshot(self, name: str, data: Dict[str, Any]) -> bool:
        """Save a snapshot to local storage"""
        try:
            # Add metadata
            data["metadata"] = {
                "name": name,
                "created_at": datetime.now().isoformat(),
                "version": "0.1.0",
            }

            # Save to file
            snapshot_file = self.snapshots_dir / f"{name}.json"
            with open(snapshot_file, "w") as f:
                json.dump(data, f, indent=2)

            return True
        except Exception as e:
            print(f"Error saving snapshot: {e}")
            return False

    def load_snapshot(self, name: str) -> Dict[str, Any]:
        """Load a snapshot from local storage"""
        try:
            snapshot_file = self.snapshots_dir / f"{name}.json"
            if not snapshot_file.exists():
                return {}

            with open(snapshot_file, "r") as f:
                return json.load(f)
        except Exception as e:
            print(f"Error loading snapshot: {e}")
            return {}

    def list_snapshots(self) -> List[Dict[str, Any]]:
        """List all available snapshots"""
        snapshots = []
        try:
            for snapshot_file in self.snapshots_dir.glob("*.json"):
                with open(snapshot_file, "r") as f:
                    data = json.load(f)
                    if "metadata" in data:
                        snapshots.append(
                            {
                                "name": data["metadata"]["name"],
                                "created_at": data["metadata"]["created_at"],
                                "file": str(snapshot_file),
                            }
                        )
        except Exception as e:
            print(f"Error listing snapshots: {e}")

        return sorted(snapshots, key=lambda x: x["created_at"], reverse=True)

    def delete_snapshot(self, name: str) -> bool:
        """Delete a snapshot"""
        try:
            snapshot_file = self.snapshots_dir / f"{name}.json"
            if snapshot_file.exists():
                snapshot_file.unlink()
                return True
            return False
        except Exception as e:
            print(f"Error deleting snapshot: {e}")
            return False


# Global storage instance
storage = LocalStorage()



"""Basic tests for DevEnv Manager"""
import pytest
import tempfile
from pathlib import Path
from unittest.mock import patch, MagicMock
from devenv.cli.main import cli
from click.testing import CliRunner


def test_cli_help():
    """Test that CLI shows help"""
    runner = CliRunner()
    result = runner.invoke(cli, ['--help'])
    assert result.exit_code == 0
    assert 'DevEnv Manager' in result.output


def test_cli_version():
    """Test CLI version command"""
    runner = CliRunner()
    result = runner.invoke(cli, ['--version'])
    assert result.exit_code == 0


def test_init_command():
    """Test init command"""
    runner = CliRunner()
    with runner.isolated_filesystem():
        result = runner.invoke(cli, ['init'])
        assert result.exit_code == 0


def test_list_command_empty():
    """Test list command with no environments"""
    runner = CliRunner()
    with runner.isolated_filesystem():
        # Mock storage directly in the main module where it's used
        with patch('devenv.cli.main.storage') as mock_storage:
            mock_storage.list_snapshots.return_value = []
            
            result = runner.invoke(cli, ['list'])
            assert result.exit_code == 0
            assert 'No environments found' in result.output


def test_list_command_with_data():
    """Test list command with existing environments"""
    runner = CliRunner()
    with runner.isolated_filesystem():
        # Mock storage with sample data
        with patch('devenv.cli.main.storage') as mock_storage:
            mock_storage.list_snapshots.return_value = [
                {
                    'name': 'test-env',
                    'created_at': '2025-05-23T20:30:00',
                    'file': '/tmp/test-env.json'
                }
            ]
            
            result = runner.invoke(cli, ['list'])
            assert result.exit_code == 0
            assert 'Available Environments' in result.output
            assert 'test-env' in result.output


def test_status_command():
    """Test status command"""
    runner = CliRunner()
    result = runner.invoke(cli, ['status'])
    assert result.exit_code == 0



"""
Environment restoration utilities for DevEnv Manager
"""

import subprocess
from pathlib import Path
from typing import Any, Dict, List

from rich.console import Console

console = Console()


class EnvironmentRestorer:
    def __init__(self):
        self.home_dir = Path.home()
        self.dry_run = False

    def restore_environment(
        self, data: Dict[str, Any], dry_run: bool = False
    ) -> bool:
        """Restore a complete environment from snapshot data"""
        self.dry_run = dry_run

        if dry_run:
            console.print(
                "[yellow]🔍 DRY RUN MODE - No changes will be " "made[/yellow]"
            )

        success = True

        # Restore packages
        if "packages" in data:
            success &= self._restore_packages(data["packages"])

        # Restore dotfiles
        if "dotfiles" in data:
            success &= self._restore_dotfiles(data["dotfiles"])

        # Restore VS Code extensions
        if "vscode_extensions" in data:
            success &= self._restore_vscode_extensions(
                data["vscode_extensions"]
            )

        return success

    def _restore_packages(self, packages: Dict[str, List[str]]) -> bool:
        """Restore system packages"""
        console.print("[bold blue]📦 Restoring packages...[/bold blue]")

        success = True

        # APT packages
        if packages.get("apt"):
            console.print(
                f"Installing {len(packages['apt'])} APT " "packages..."
            )
            success &= self._install_apt_packages(packages["apt"])

        # Snap packages
        if packages.get("snap"):
            console.print(
                f"Installing {len(packages['snap'])} Snap " "packages..."
            )
            success &= self._install_snap_packages(packages["snap"])

        # Flatpak packages
        if packages.get("flatpak"):
            console.print(
                f"Installing {len(packages['flatpak'])} Flatpak " "packages..."
            )
            success &= self._install_flatpak_packages(packages["flatpak"])

        # PIP packages
        if packages.get("pip"):
            console.print(
                f"Installing {len(packages['pip'])} PIP " "packages..."
            )
            success &= self._install_pip_packages(packages["pip"])

        return success

    def _install_apt_packages(self, packages: List[str]) -> bool:
        """Install APT packages"""
        if not packages:
            return True

        # Filter out packages that are already installed
        to_install = []
        for package in packages:
            if not self._is_apt_package_installed(package):
                to_install.append(package)

        if not to_install:
            console.print(
                "[green]✓ All APT packages already " "installed[/green]"
            )
            return True

        console.print(f"Installing {len(to_install)} new APT packages...")

        if self.dry_run:
            preview_pkgs = " ".join(to_install[:10])
            if len(to_install) > 10:
                preview_pkgs += "..."
            console.print(f"[dim]Would install: {preview_pkgs}[/dim]")
            return True

        try:
            # Update package list first
            subprocess.run(
                ["sudo", "apt", "update"], check=True, capture_output=True
            )

            # Install packages in batches to avoid command line too long
            batch_size = 50
            for i in range(0, len(to_install), batch_size):
                batch = to_install[i: i + batch_size]
                cmd = ["sudo", "apt", "install", "-y"] + batch
                result = subprocess.run(cmd, capture_output=True, text=True)

                if result.returncode != 0:
                    console.print(
                        f"[red]Failed to install some APT "
                        f"packages: {result.stderr}[/red]"
                    )
                    return False

            console.print(
                "[green]✓ APT packages installed " "successfully[/green]"
            )
            return True

        except subprocess.CalledProcessError as e:
            console.print(f"[red]Error installing APT packages: {e}[/red]")
            return False

    def _install_snap_packages(self, packages: List[str]) -> bool:
        """Install Snap packages"""
        if not packages:
            return True

        if self.dry_run:
            console.print(
                f"[dim]Would install {len(packages)} snap " "packages[/dim]"
            )
            return True

        try:
            for package in packages:
                if not self._is_snap_package_installed(package):
                    console.print(f"Installing snap: {package}")
                    subprocess.run(
                        ["sudo", "snap", "install", package],
                        check=True,
                        capture_output=True,
                    )

            console.print(
                "[green]✓ Snap packages installed " "successfully[/green]"
            )
            return True

        except subprocess.CalledProcessError as e:
            console.print(
                f"[red]Error installing Snap packages: " f"{e}[/red]"
            )
            return False

    def _install_flatpak_packages(self, packages: List[str]) -> bool:
        """Install Flatpak packages"""
        if not packages:
            return True

        if self.dry_run:
            console.print(
                f"[dim]Would install {len(packages)} flatpak " "packages[/dim]"
            )
            return True

        try:
            for package in packages:
                console.print(f"Installing flatpak: {package}")
                subprocess.run(
                    ["flatpak", "install", "-y", package],
                    check=True,
                    capture_output=True,
                )

            console.print(
                "[green]✓ Flatpak packages installed " "successfully[/green]"
            )
            return True

        except subprocess.CalledProcessError as e:
            console.print(
                f"[red]Error installing Flatpak packages: " f"{e}[/red]"
            )
            return False

    def _install_pip_packages(self, packages: List[str]) -> bool:
        """Install PIP packages"""
        if not packages:
            return True

        if self.dry_run:
            console.print(
                f"[dim]Would install {len(packages)} pip " "packages[/dim]"
            )
            return True

        try:
            # Install packages
            cmd = ["pip", "install"] + packages
            subprocess.run(cmd, check=True, capture_output=True)

            console.print(
                "[green]✓ PIP packages installed " "successfully[/green]"
            )
            return True

        except subprocess.CalledProcessError as e:
            console.print(f"[red]Error installing PIP packages: " f"{e}[/red]")
            return False

    def _restore_dotfiles(self, dotfiles: Dict[str, str]) -> bool:
        """Restore dotfiles"""
        if not dotfiles:
            return True

        console.print(
            f"[bold blue]📝 Restoring {len(dotfiles)} "
            "dotfiles...[/bold blue]"
        )

        if self.dry_run:
            console.print(
                f"[dim]Would restore: " f"{', '.join(dotfiles.keys())}[/dim]"
            )
            return True

        try:
            for filename, content in dotfiles.items():
                file_path = self.home_dir / filename

                # Create directory if needed
                file_path.parent.mkdir(parents=True, exist_ok=True)

                # Backup existing file
                if file_path.exists():
                    backup_path = file_path.with_suffix(
                        file_path.suffix + ".devenv-backup"
                    )
                    file_path.rename(backup_path)
                    console.print(
                        f"Backed up existing {filename} to "
                        f"{backup_path.name}"
                    )

                # Write new content
                with open(file_path, "w") as f:
                    f.write(content)

                console.print(f"✓ Restored {filename}")

            console.print("[green]✓ Dotfiles restored successfully[/green]")
            return True

        except Exception as e:
            console.print(f"[red]Error restoring dotfiles: {e}[/red]")
            return False

    def _restore_vscode_extensions(self, extensions: List[str]) -> bool:
        """Restore VS Code extensions"""
        if not extensions:
            return True

        console.print(
            f"[bold blue]🔌 Restoring {len(extensions)} VS Code "
            "extensions...[/bold blue]"
        )

        if self.dry_run:
            preview_ext = ", ".join(extensions[:5])
            if len(extensions) > 5:
                preview_ext += "..."
            console.print(f"[dim]Would install: {preview_ext}[/dim]")
            return True

        try:
            for extension in extensions:
                console.print(f"Installing extension: {extension}")
                subprocess.run(
                    ["code", "--install-extension", extension],
                    check=True,
                    capture_output=True,
                )

            console.print(
                "[green]✓ VS Code extensions installed " "successfully[/green]"
            )
            return True

        except subprocess.CalledProcessError as e:
            console.print(
                f"[red]Error installing VS Code extensions: " f"{e}[/red]"
            )
            return False

    def _is_apt_package_installed(self, package: str) -> bool:
        """Check if APT package is installed"""
        try:
            result = subprocess.run(
                ["dpkg", "-l", package],
                capture_output=True,
                text=True,
                check=True,
            )
            return "ii" in result.stdout
        except subprocess.CalledProcessError:
            return False

    def _is_snap_package_installed(self, package: str) -> bool:
        """Check if Snap package is installed"""
        try:
            result = subprocess.run(
                ["snap", "list", package],
                capture_output=True,
                text=True,
                check=True,
            )
            return package in result.stdout
        except subprocess.CalledProcessError:
            return False


# Global restorer instance
restorer = EnvironmentRestorer()



"""
Git-based synchronization for DevEnv Manager
"""

import json
import subprocess
from typing import Any, Dict, List, Optional

import click
from rich.console import Console

from ..core.config import config
from ..storage.local import storage

console = Console()


class GitSync:
    def __init__(self):
        self.sync_dir = config.config_dir / "sync"
        self.repo = None
        self.remote_url = None

    def setup_sync(self, remote_url: str, branch: str = "main") -> bool:
        """Setup git sync with a remote repository"""
        try:
            # Create sync directory
            self.sync_dir.mkdir(exist_ok=True)

            # Initialize git repo if needed
            if not (self.sync_dir / ".git").exists():
                subprocess.run(
                    ["git", "init"],
                    cwd=self.sync_dir,
                    check=True,
                    capture_output=True,
                )

                # Create initial commit
                readme_content = """# DevEnv Manager Sync Repository

This repository contains synchronized development environments.
Generated automatically by DevEnv Manager.

## Contents
- `environments/` - Environment snapshots
- `metadata.json` - Sync metadata

Do not edit these files manually.
"""
                with open(self.sync_dir / "README.md", "w") as f:
                    f.write(readme_content)

                # Create environments directory
                (self.sync_dir / "environments").mkdir(exist_ok=True)

                # Create metadata file
                metadata = {
                    "version": "0.1.0",
                    "created_by": "devenv-manager",
                    "last_sync": None,
                    "environments": {},
                }
                with open(self.sync_dir / "metadata.json", "w") as f:
                    json.dump(metadata, f, indent=2)

                # Initial commit
                subprocess.run(
                    ["git", "add", "."],
                    cwd=self.sync_dir,
                    check=True,
                    capture_output=True,
                )
                subprocess.run(
                    [
                        "git",
                        "commit",
                        "-m",
                        "Initial DevEnv Manager sync setup",
                    ],
                    cwd=self.sync_dir,
                    check=True,
                    capture_output=True,
                )

            # Add remote if not exists
            try:
                subprocess.run(
                    ["git", "remote", "add", "origin", remote_url],
                    cwd=self.sync_dir,
                    check=True,
                    capture_output=True,
                )
            except subprocess.CalledProcessError:
                # Remote might already exist, try to set URL
                subprocess.run(
                    ["git", "remote", "set-url", "origin", remote_url],
                    cwd=self.sync_dir,
                    check=True,
                    capture_output=True,
                )

            # Set branch
            subprocess.run(
                ["git", "branch", "-M", branch],
                cwd=self.sync_dir,
                check=True,
                capture_output=True,
            )

            self.remote_url = remote_url

            # Save sync config
            config.settings["sync"] = {
                "enabled": True,
                "remote_url": remote_url,
                "branch": branch,
                "last_sync": None,
            }
            config.save_config()

            console.print(
                f"[green]✓ Git sync setup successfully with "
                f"{remote_url}[/green]"
            )
            return True

        except subprocess.CalledProcessError as e:
            console.print(f"[red]✗ Git sync setup failed: {e}[/red]")
            return False
        except Exception as e:
            console.print(f"[red]✗ Sync setup error: {e}[/red]")
            return False

    def push_environments(
        self, environment_names: Optional[List[str]] = None
    ) -> bool:
        """Push environments to remote repository"""
        if not self._check_sync_setup():
            return False

        try:
            # Get environments to sync
            if environment_names is None:
                snapshots = storage.list_snapshots()
                environment_names = [s["name"] for s in snapshots]

            if not environment_names:
                console.print("[yellow]No environments to sync[/yellow]")
                return True

            console.print(
                f"[blue]📤 Pushing {len(environment_names)} "
                "environments...[/blue]"
            )

            # Copy environments to sync directory
            env_dir = self.sync_dir / "environments"
            env_dir.mkdir(exist_ok=True)

            synced_count = 0
            for env_name in environment_names:
                data = storage.load_snapshot(env_name)
                if data:
                    env_file = env_dir / f"{env_name}.json"
                    with open(env_file, "w") as f:
                        json.dump(data, f, indent=2)
                    synced_count += 1

            # Update metadata
            self._update_metadata(environment_names)

            # Git operations
            subprocess.run(
                ["git", "add", "."],
                cwd=self.sync_dir,
                check=True,
                capture_output=True,
            )

            # Check if there are changes to commit
            result = subprocess.run(
                ["git", "status", "--porcelain"],
                cwd=self.sync_dir,
                capture_output=True,
                text=True,
            )

            if result.stdout.strip():
                # Commit changes
                commit_msg = f"Sync {synced_count} environments"
                subprocess.run(
                    ["git", "commit", "-m", commit_msg],
                    cwd=self.sync_dir,
                    check=True,
                    capture_output=True,
                )

                # Push to remote
                subprocess.run(
                    ["git", "push", "-u", "origin", "main"],
                    cwd=self.sync_dir,
                    check=True,
                    capture_output=True,
                )

                console.print(
                    f"[green]✓ Successfully pushed "
                    f"{synced_count} environments[/green]"
                )
            else:
                console.print("[yellow]No changes to sync[/yellow]")

            return True

        except subprocess.CalledProcessError as e:
            console.print(f"[red]✗ Push failed: {e}[/red]")
            return False
        except Exception as e:
            console.print(f"[red]✗ Push error: {e}[/red]")
            return False

    def pull_environments(self) -> bool:
        """Pull environments from remote repository"""
        if not self._check_sync_setup():
            return False

        try:
            console.print(
                "[blue]📥 Pulling environments from " "remote...[/blue]"
            )

            # Pull from remote
            subprocess.run(
                ["git", "pull", "origin", "main"],
                cwd=self.sync_dir,
                check=True,
                capture_output=True,
            )

            # Import environments from sync directory
            env_dir = self.sync_dir / "environments"
            if not env_dir.exists():
                console.print(
                    "[yellow]No environments found in remote "
                    "repository[/yellow]"
                )
                return True

            return self._import_environments_from_sync_dir(env_dir)

        except subprocess.CalledProcessError as e:
            console.print(f"[red]✗ Pull failed: {e}[/red]")
            return False
        except Exception as e:
            console.print(f"[red]✗ Pull error: {e}[/red]")
            return False

    def _import_environments_from_sync_dir(self, env_dir) -> bool:
        """Import environments from sync directory"""
        imported_count = 0

        for env_file in env_dir.glob("*.json"):
            try:
                with open(env_file, "r") as f:
                    data = json.load(f)

                env_name = env_file.stem

                # Check if environment already exists locally
                existing_data = storage.load_snapshot(env_name)
                if existing_data:
                    if not self._should_overwrite_local_env(
                        env_name, data, existing_data
                    ):
                        continue

                # Import environment
                if storage.save_snapshot(env_name, data):
                    imported_count += 1
                    console.print(f"[green]✓ Imported {env_name}[/green]")

            except Exception as e:
                console.print(
                    f"[red]✗ Failed to import " f"{env_file.name}: {e}[/red]"
                )

        console.print(
            f"[green]✓ Successfully imported "
            f"{imported_count} environments[/green]"
        )
        return True

    def _should_overwrite_local_env(
        self, env_name, remote_data, local_data
    ) -> bool:
        """Check if local environment should be overwritten"""
        # Compare timestamps or ask user
        remote_time = remote_data.get("metadata", {}).get("created_at", "")
        local_time = local_data.get("metadata", {}).get("created_at", "")

        if remote_time <= local_time:
            console.print(
                f"[dim]Skipping {env_name} (local version is " "newer)[/dim]"
            )
            return False

        console.print(
            f"[yellow]Environment '{env_name}' already exists "
            "locally[/yellow]"
        )
        return click.confirm(f"Overwrite local version of '{env_name}'?")

    def sync_status(self) -> Dict[str, Any]:
        """Get sync status information"""
        if not self._check_sync_setup():
            return {"enabled": False}

        try:
            # Get git status
            result = subprocess.run(
                ["git", "status", "--porcelain"],
                cwd=self.sync_dir,
                capture_output=True,
                text=True,
            )

            uncommitted_changes = bool(result.stdout.strip())

            # Get last commit info
            try:
                result = subprocess.run(
                    ["git", "log", "-1", "--format=%H|%s|%ai"],
                    cwd=self.sync_dir,
                    capture_output=True,
                    text=True,
                    check=True,
                )
                last_commit_info = result.stdout.strip().split("|")
                last_commit = {
                    "hash": last_commit_info[0][:8],
                    "message": last_commit_info[1],
                    "date": last_commit_info[2],
                }
            except Exception:
                last_commit = None

            return {
                "enabled": True,
                "remote_url": (
                    self.remote_url or config.get("sync.remote_url")
                ),
                "branch": "main",
                "uncommitted_changes": uncommitted_changes,
                "last_commit": last_commit,
                "sync_dir": str(self.sync_dir),
            }

        except Exception as e:
            return {"enabled": True, "error": str(e)}

    def _check_sync_setup(self) -> bool:
        """Check if sync is properly setup"""
        if not config.get("sync.enabled"):
            console.print(
                "[red]✗ Git sync not setup. Run 'devenv sync "
                "setup <repo-url>' first[/red]"
            )
            return False

        if not self.sync_dir.exists() or not (self.sync_dir / ".git").exists():
            console.print(
                "[red]✗ Git sync directory corrupted. Run setup " "again[/red]"
            )
            return False

        return True

    def _update_metadata(self, environment_names: List[str]):
        """Update sync metadata"""
        from datetime import datetime

        metadata_file = self.sync_dir / "metadata.json"

        if metadata_file.exists():
            with open(metadata_file, "r") as f:
                metadata = json.load(f)
        else:
            metadata = {"environments": {}}

        metadata["last_sync"] = datetime.now().isoformat()

        for env_name in environment_names:
            metadata["environments"][env_name] = {
                "last_synced": datetime.now().isoformat()
            }

        with open(metadata_file, "w") as f:
            json.dump(metadata, f, indent=2)


# Global sync instance
git_sync = GitSync()



from setuptools import setup, find_packages

# Ler README com encoding UTF-8 para evitar problemas
with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="devenv-manager",
    version="0.1.2",
    description="Development Environment Manager for Linux - Backup, sync and restore your dev environment in minutes",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Bernardo Amorim Alvarenga",
    author_email="amorimbernardogame@gmail.com",
    url="https://github.com/bernardoamorimalvarenga/devenv-manager",
    
    # Links importantes que aparecem no PyPI
    project_urls={
        "Bug Reports": "https://github.com/bernardoamorimalvarenga/devenv-manager/issues",
        "Source": "https://github.com/bernardoamorimalvarenga/devenv-manager",
        "Documentation": "https://github.com/bernardoamorimalvarenga/devenv-manager#readme",
        "Changelog": "https://github.com/bernardoamorimalvarenga/devenv-manager/blob/main/CHANGELOG.md",
    },
    
    packages=find_packages(where="src"),
    package_dir={"": "src"},
    
    install_requires=[
        "click>=8.0.0",
        "rich>=13.0.0",
        "pyyaml>=6.0",
        "gitpython>=3.1.0",
        "psutil>=5.9.0",
    ],
    
    extras_require={
        "dev": [
            "pytest>=7.0.0",
            "pytest-cov>=4.0.0",
            "black>=23.0.0",
            "isort>=5.12.0",
            "flake8>=6.0.0",
            "autoflake>=2.0.0",
        ]
    },
    
    entry_points={
        "console_scripts": [
            "devm=devenv.cli.main:cli",
        ],
    },
    
    python_requires=">=3.8",
    
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Topic :: Software Development",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: System :: Systems Administration",
        "Topic :: System :: Archiving :: Backup",
        "Topic :: Utilities",
        "Environment :: Console",
    ],
    
    keywords="linux development environment backup sync cli devops dotfiles packages git automation",
    
    # Metadados adicionais para melhor descoberta
    platforms=["Linux"],
    zip_safe=False,
)
